==> speechanalytic-billing_0.1.4_database.tar.gz: database/mssql/procedures/GetPlatformCurrentBatches.sql <==
SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTA FUNCION RETORNA LAS TARIFAS SETTEADAS DE UNA CUENTA O SUBCUENTA
CREATE PROCEDURE [dbo].[GetPlatformCurrentBatches] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(100) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- ESTADOS DE LOS BATCHASUMMARY:
	-- CREATED: ESTADO POR DEFECTO AL CREAR EL BATCH
	-- RUNNING: ESTADO CUANDO EL BATCH TIENE 1 O MAS DETALLES (BATCHDETAILED)
	-- PAUSED_BY_USER: ESTADO CUANDO EL BATCH ES PAUSADO POR EL USUARIO POR ALGUN MOTIVO
	-- PAUSED_BY_INSUFFICIENT_BALANCE: ESTADO CUANDO EL BATCH ES PAUSADO POR EL SISTEMA POR NO TENER SALDO DISPONIBLE
	-- CLOSED: ESTADO CUANDO EL BATCH TERMINO SIN PROBLEMA NI INTERVENSION
	-- CANCELLED_BY_TIMEOUT: ESTADO CUANDO EL BATCH ES TERMINADO POR EL SISTEMA POR SUPERAR 72 HORAS DESDE LA PAUSA
	-- CANCELLED_BY_USER: ESTADO CUANDO EL BATCH ES TERMINADO POR EL USUARIO POR ALGUN MOTIVO
	
	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- SCRIPT PARA RECUPERAR LOS SALDOS ACTUALES DE LA PLATAFORMA
		SELECT AVB.[InstallationId],
				AVB.[AccountId],
				AVB.[SubAccountId],
				AVB.[VCC],
				AVB.[SystemService],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
			WHERE AVB.[InstallationId] = @InstallationId
				AND AVB.[VCC] IN (
					SELECT ASAVST.[VCC] 
						FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
						WHERE ASAVST.[InstallationId] = AVB.[InstallationId]
							-- AND ASAVST.[IsFinished] = 0
							AND ASAVST.[Status] NOT LIKE 'FINISHED%'
				)

		-- DEFINIMOS EL TIEMPO EN HORAS PARA CERRAR UN BATCH AUTOMATICAMENTE (SI SE MODIFICA, MODIFICAR TAMBIEN EN EL SP GETBATCHBALANCEDETAIL)
		DECLARE @HOURS_TO_CLOSE_PAUSED_BATCH_SUMMARY DECIMAL(18,5) = 72

		-- HACEMO UN UPDATE A LA TABLA SUMMARY PARA MARCAR COMO CANCELLED_BY_TIMEOUT LOS BATCHES PAUSADOS CON 72 HORAS O MAS
		UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction] 
			SET [Status] = 'CANCELLED_BY_TIMEOUT',
				[LastModifiedByUserId] = 'system',
				[LasModifiedDate] = CURRENT_TIMESTAMP,
				[TimeStamp] =  CURRENT_TIMESTAMP
			WHERE [Status] LIKE 'PAUSED%' 
				AND GETUTCDATE() >= DATEADD(HOUR , @HOURS_TO_CLOSE_PAUSED_BATCH_SUMMARY , [LasModifiedDate]) 

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS BATCH CREATED QUE TIENEN DETALLE
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SCRIPT PARA RETORNAR BATCHES CON ESTADO CREATED QUE TIENEN 1 O MAS DETALLES
			SELECT ASAVST.[BatchId],
					ASAVST.[VCC],
					ASAVST.[SystemService]
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
				LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT 
					ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
						AND ASAVDT.[VCC] = ASAVST.[VCC]
						AND ASAVDT.[BatchId] = ASAVST.[BatchId]
						AND ASAVDT.[SystemService] = ASAVST.[SystemService]
				WHERE ASAVST.[InstallationId] = @InstallationId
					--AND ASAVST.[IsFinished] = 0 <--HUBO UN PROBLEMA CUANDO UN COGNITIVO LLENO DE FALLIDOS AL PRIMER LANZAMIENTO
					AND ASAVST.[Status] = 'CREATED'
					AND ASAVDT.[BatchId] IS NOT NULL
				GROUP BY ASAVST.[InstallationId],
					ASAVST.[BatchId],
					ASAVST.[VCC],
					ASAVST.[SystemService],
					ASAVST.[CreatedDate]
				ORDER BY ASAVST.[CreatedDate] ASC

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL BACTH ACTUAL QUE ESTAMOS RECORRIENDO
		DECLARE @CURRENT_BATCH_ID VARCHAR(50) = ''
		DECLARE @CURRENT_BATCH_VCC VARCHAR(50) = ''
		DECLARE @CURRENT_BATCH_SYSTEM_SERVICE VARCHAR(50) = ''

		-- ABRIMOS Y RECORREMOS EL ARRAY DE PRODUCTOS
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_BATCH_ID, @CURRENT_BATCH_VCC, @CURRENT_BATCH_SYSTEM_SERVICE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			-- ACTUALIZAMOS A ESTADO RUNNING CADA BATCH DE ESTADO CREATED QUE YA TIENE 1 REGISTRO EN DETAILED
			UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction]
				SET [Status] = 'RUNNING'
				WHERE [InstallationId] = @InstallationId
					AND [BatchId] = @CURRENT_BATCH_ID
					AND [VCC] = @CURRENT_BATCH_VCC
					AND [SystemService] = @CURRENT_BATCH_SYSTEM_SERVICE

			FETCH NEXT FROM MY_CURSOR INTO @CURRENT_BATCH_ID, @CURRENT_BATCH_VCC, @CURRENT_BATCH_SYSTEM_SERVICE
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- SCRIPT PARA RECUPERAR LOS BATCH ABIERTOS, EL ACUMULADO DE LOS DETALLES CERRADOS Y EL RETENIDO GENERAL DEL BATCH
		SELECT ASAVST.[InstallationId],
				ASAVST.[BatchId],
				ASAVST.[CreatedDate],
				ASAVST.[BatchTypeName],
				ASAVST.[BatchTypeId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[SystemService],
				SUM(ASAVDT.[Rate]) 'RATED',
				ASAVBM.[Amount] 'RETAINED',
				COUNT(ASAVDT.[BatchId]) 'DETAILS',
				ASAVST.[Status],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
			LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT 
				ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVDT.[VCC] = ASAVST.[VCC]
					AND ASAVDT.[BatchId] = ASAVST.[BatchId]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			INNER JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
				ON ASAVBM.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVBM.[VCC] = ASAVST.[VCC]
					AND ASAVBM.BatchId = ASAVST.[BatchId]
					AND ASAVBM.[Wallet] = ASAVST.[SystemService]
			INNER JOIN [dbo].[AccountVCCBalance] AVB  (NOLOCK) 
				ON AVB.[InstallationId] = ASAVST.[InstallationId]
					AND AVB.[AccountId] = ASAVBM.[AccountId]
					AND AVB.[VCC] = ASAVST.[VCC]
					AND AVB.[SystemService] = ASAVST.[SystemService]
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND (ASAVST.[Status] IN ('CREATED', 'RUNNING')
					OR ASAVST.[Status] LIKE 'CANCELLED%')
				--AND ASAVST.[IsFinished] = 0
				--AND ASAVDT.[IsValid] = 1
				--AND ASAVDT.[IsFinished]= 1
				AND ASAVBM.[Type] = 'RETAIN'
				AND GETUTCDATE() <= DATEADD(HOUR , 98 , CASE WHEN ASAVST.[LasModifiedDate] IS NULL THEN GETUTCDATE() ELSE ASAVST.[LasModifiedDate] END) 
			GROUP BY ASAVST.[InstallationId],
				ASAVST.[BatchId],
				ASAVST.[BatchTypeName],
				ASAVST.[BatchTypeId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[SystemService],
				ASAVST.[CreatedDate],
				ASAVBM.[Amount],
				ASAVST.[Status],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			ORDER BY ASAVST.[CreatedDate] DESC

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

==> speechanalytic-billing_1.0.1_database.tar.gz: database/mssql/procedures/GetPlatformCurrentBatches.sql <==
SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTA FUNCION RETORNA LAS TARIFAS SETTEADAS DE UNA CUENTA O SUBCUENTA
CREATE PROCEDURE [dbo].[GetPlatformCurrentBatches] (
	@PlatformHash VARCHAR(100) = NULL,
	@InstallationId VARCHAR(100) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- ESTADOS DE LOS BATCHASUMMARY:
	-- CREATED: ESTADO POR DEFECTO AL CREAR EL BATCH
	-- RUNNING: ESTADO CUANDO EL BATCH TIENE 1 O MAS DETALLES (BATCHDETAILED)
	-- PAUSED_BY_USER: ESTADO CUANDO EL BATCH ES PAUSADO POR EL USUARIO POR ALGUN MOTIVO
	-- PAUSED_BY_INSUFFICIENT_BALANCE: ESTADO CUANDO EL BATCH ES PAUSADO POR EL SISTEMA POR NO TENER SALDO DISPONIBLE
	-- CLOSED: ESTADO CUANDO EL BATCH TERMINO SIN PROBLEMA NI INTERVENSION
	-- CANCELLED_BY_PAUSED_TIMEOUT: ESTADO CUANDO EL BATCH ES TERMINADO POR EL SISTEMA POR SUPERAR 72 HORAS DESDE LA PAUSA
	-- CANCELLED_BY_USER: ESTADO CUANDO EL BATCH ES TERMINADO POR EL USUARIO POR ALGUN MOTIVO
	
	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- SCRIPT PARA RECUPERAR LOS SALDOS ACTUALES DE LA PLATAFORMA
		SELECT AVB.[InstallationId],
				AVB.[AccountId],
				AVB.[SubAccountId],
				AVB.[VCC],
				AVB.[SystemService],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			FROM [dbo].[AccountVCCBalance] AVB (NOLOCK)
			WHERE AVB.[InstallationId] = @InstallationId
				AND AVB.[VCC] IN (
					SELECT ASAVST.[VCC] 
						FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
						WHERE ASAVST.[InstallationId] = AVB.[InstallationId]
							-- AND ASAVST.[IsFinished] = 0
							AND ASAVST.[Status] NOT LIKE 'FINISHED%'
				)

		-- DEFINIMOS EL TIEMPO EN HORAS PARA CERRAR UN BATCH AUTOMATICAMENTE (SI SE MODIFICA, MODIFICAR TAMBIEN EN EL SP GETBATCHBALANCEDETAIL)
		DECLARE @HOURS_TO_CLOSE_PAUSED_BATCH_SUMMARY DECIMAL(18,5) = 72
		DECLARE @HOURS_TO_CLOSE_RUNNING_BATCH_SUMMARY DECIMAL(18,5) = 120

		-- HACEMO UN UPDATE A LA TABLA SUMMARY PARA MARCAR COMO CANCELLED_BY_PAUSED_TIMEOUT LOS BATCHES PAUSADOS CON 72 HORAS O MAS
		UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction] 
			SET [Status] = 'CANCELLED_BY_PAUSED_TIMEOUT',
				[LastModifiedByUserId] = 'system',
				[LasModifiedDate] = GETUTCDATE(),
				[TimeStamp] =  GETUTCDATE()
			WHERE [Status] LIKE 'PAUSED%' 
				AND GETUTCDATE() >= DATEADD(HOUR ,@HOURS_TO_CLOSE_PAUSED_BATCH_SUMMARY ,ISNULL([LasModifiedDate], [TimeStamp])) 

		-- HACEMO UN UPDATE A LA TABLA SUMMARY PARA MARCAR COMO CANCELLED_BY_RUNNING_TIMEOUT LOS BATCHES COLGADOS EN RUNNING O CREATED CON 120 HORAS O MAS
		UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction] 
			SET [Status] = 'CANCELLED_BY_RUNNING_TIMEOUT',
				[LastModifiedByUserId] = 'system',
				[LasModifiedDate] = GETUTCDATE(),
				[TimeStamp] =  GETUTCDATE()
			WHERE (
					[Status] LIKE 'RUNNING%' 
					OR
					[Status] LIKE 'CREATED%' 
				)
				AND GETUTCDATE() >= DATEADD(HOUR ,@HOURS_TO_CLOSE_RUNNING_BATCH_SUMMARY ,ISNULL([LasModifiedDate], [CreatedDate])) 

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS BATCH CREATED QUE TIENEN DETALLE
		DECLARE MY_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SCRIPT PARA RETORNAR BATCHES CON ESTADO CREATED QUE TIENEN 1 O MAS DETALLES
			SELECT ASAVST.[BatchId],
					ASAVST.[VCC],
					ASAVST.[SystemService]
				FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK) 
				LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT 
					ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
						AND ASAVDT.[VCC] = ASAVST.[VCC]
						AND ASAVDT.[BatchId] = ASAVST.[BatchId]
						AND ASAVDT.[SystemService] = ASAVST.[SystemService]
				WHERE ASAVST.[InstallationId] = @InstallationId
					--AND ASAVST.[IsFinished] = 0 <--HUBO UN PROBLEMA CUANDO UN COGNITIVO LLENO DE FALLIDOS AL PRIMER LANZAMIENTO
					AND ASAVST.[Status] = 'CREATED'
					AND ASAVDT.[BatchId] IS NOT NULL
				GROUP BY ASAVST.[InstallationId],
					ASAVST.[BatchId],
					ASAVST.[VCC],
					ASAVST.[SystemService],
					ASAVST.[CreatedDate]
				ORDER BY ASAVST.[CreatedDate] ASC

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL BACTH ACTUAL QUE ESTAMOS RECORRIENDO
		DECLARE @CURRENT_BATCH_ID VARCHAR(50) = ''
		DECLARE @CURRENT_BATCH_VCC VARCHAR(50) = ''
		DECLARE @CURRENT_BATCH_SYSTEM_SERVICE VARCHAR(50) = ''

		-- ABRIMOS Y RECORREMOS EL ARRAY DE PRODUCTOS
		OPEN MY_CURSOR
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_BATCH_ID, @CURRENT_BATCH_VCC, @CURRENT_BATCH_SYSTEM_SERVICE
		WHILE @@FETCH_STATUS = 0
		BEGIN 
			-- ACTUALIZAMOS A ESTADO RUNNING CADA BATCH DE ESTADO CREATED QUE YA TIENE 1 REGISTRO EN DETAILED
			UPDATE [dbo].[AccountSubAccountVCCSummaryTransaction]
				SET [Status] = 'RUNNING'
				WHERE [InstallationId] = @InstallationId
					AND [BatchId] = @CURRENT_BATCH_ID
					AND [VCC] = @CURRENT_BATCH_VCC
					AND [SystemService] = @CURRENT_BATCH_SYSTEM_SERVICE

			FETCH NEXT FROM MY_CURSOR INTO @CURRENT_BATCH_ID, @CURRENT_BATCH_VCC, @CURRENT_BATCH_SYSTEM_SERVICE
		END
		CLOSE MY_CURSOR
		DEALLOCATE MY_CURSOR

		-- SCRIPT PARA RECUPERAR LOS BATCH ABIERTOS, EL ACUMULADO DE LOS DETALLES CERRADOS Y EL RETENIDO GENERAL DEL BATCH
		SELECT ASAVST.[InstallationId],
				ASAVST.[BatchId],
				ASAVST.[CreatedDate],
				ASAVST.[BatchTypeName],
				ASAVST.[BatchTypeId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[SystemService],
				SUM(ASAVDT.[Rate])							'RATED',
				ASAVBM.[Amount]								'RETAINED',
				COUNT(ASAVDT.[BatchId])						'DETAILS',
				ASAVST.[Status],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			FROM [dbo].[AccountSubAccountVCCSummaryTransaction] ASAVST (NOLOCK)
			LEFT JOIN [dbo].[AccountSubAccountVCCDetailedTransaction] ASAVDT (NOLOCK)
				ON ASAVDT.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVDT.[VCC] = ASAVST.[VCC]
					AND ASAVDT.[BatchId] = ASAVST.[BatchId]
					AND ASAVDT.[SystemService] = ASAVST.[SystemService]
			INNER JOIN [dbo].[AccountSubAccountVCCBalanceMovement] ASAVBM (NOLOCK) 
				ON ASAVBM.[InstallationId] = ASAVST.[InstallationId]
					AND ASAVBM.[VCC] = ASAVST.[VCC]
					AND ASAVBM.BatchId = ASAVST.[BatchId]
					AND ASAVBM.[Wallet] = ASAVST.[SystemService]
			INNER JOIN [dbo].[AccountVCCBalance] AVB  (NOLOCK) 
				ON AVB.[InstallationId] = ASAVST.[InstallationId]
					AND AVB.[AccountId] = ASAVBM.[AccountId]
					AND AVB.[VCC] = ASAVST.[VCC]
					AND AVB.[SystemService] = ASAVST.[SystemService]
			WHERE ASAVST.[InstallationId] = @InstallationId
				AND (
					ASAVST.[Status] IN ('CREATED', 'RUNNING')
					OR 
					ASAVST.[Status] LIKE 'CANCELLED%'
				)
				--AND ASAVST.[IsFinished] = 0
				--AND ASAVDT.[IsValid] = 1
				--AND ASAVDT.[IsFinished]= 1
				AND ASAVBM.[Type] = 'RETAIN'
				-- AND GETUTCDATE() <= DATEADD(HOUR , 98 , CASE WHEN ASAVST.[LasModifiedDate] IS NULL THEN GETUTCDATE() ELSE ASAVST.[LasModifiedDate] END) 
				AND GETUTCDATE() <= DATEADD(HOUR , 98 , CASE WHEN ASAVST.[LasModifiedDate] IS NULL THEN ASAVST.[CreatedDate] ELSE ASAVST.[LasModifiedDate] END) 
			GROUP BY ASAVST.[InstallationId],
				ASAVST.[BatchId],
				ASAVST.[BatchTypeName],
				ASAVST.[BatchTypeId],
				ASAVBM.[AccountId],
				ASAVBM.[SubAccountId],
				ASAVST.[VCC],
				ASAVST.[SystemService],
				ASAVST.[CreatedDate],
				ASAVBM.[Amount],
				ASAVST.[Status],
				AVB.[CountableBalance],
				AVB.[AvailableBalance],
				AVB.[RetainedBalance]
			ORDER BY ASAVST.[CreatedDate] DESC

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
